from __future__ import annotations

from prettyqt import widgets
from prettyqt.qt import QtWidgets


class StyleOptionToolButton(widgets.StyleOptionComplex, QtWidgets.QStyleOptionToolButton):
    pass
