from __future__ import annotations

from prettyqt import widgets
from prettyqt.qt import QtWidgets


class StyleOptionProgressBar(widgets.StyleOptionMixin, QtWidgets.QStyleOptionProgressBar):
    pass
