from __future__ import annotations

from prettyqt import widgets
from prettyqt.qt import QtWidgets


class StyleOptionTabWidgetFrame(
    widgets.StyleOptionMixin, QtWidgets.QStyleOptionTabWidgetFrame
):
    pass
