from __future__ import annotations

from prettyqt import widgets
from prettyqt.qt import QtWidgets


class StyleOptionSizeGrip(
    widgets.StyleOptionComplexMixin, QtWidgets.QStyleOptionSizeGrip
):
    pass
